/*! \file translations_dbg.h
 * \brief Auxiliary functions for debugging and testing (complementary to translations.h)
 * 
 * The following functions evaluate the SSWFs the same way as
 * certain functions in translations.h, but they leave them
 * as they are, without rearranging them into the translation
 * operators.
 */
#ifndef TRANSLATIONS_DBG_H

#include "translations.h"

/// Evaluates scalar spherical wavefunctions at a point (spherical coordinates)
/**
 * \f[
 *   \phi_l^m (kr, \theta, \varphi) = h_l(kr) \rawLeg{l}{m}(\cos \theta) e^{im\varphi} 
 * \f]
 *
 * The wavefunctions are evaluated up to the degree `2 * c->lMax + 1`.
 *
 * Currently, THIS ALWAYS USES NON-NORMALISED LEGENDRE FUNCTIONS (TODO FIXME?).
 */
int qpms_trans_calculator_test_sswf(const qpms_trans_calculator *c,
    _Complex double *dest, ///< Result array, length `qpms_lMax2nelem_sc(c->lMax)`
    const csph_t kdlj, ///< Evaluation point (incl. wavenumber)
    const  qpms_bessel_t J);

/// Evaluates scalar spherical wavefunctions at a point (cartesian coordinates)
/**
 * Currently, THIS ALWAYS USES NON-NORMALISED LEGENDRE FUNCTIONS (TODO FIXME?).
 *
 * \see qpms_trans_calculator_test_sswf()
 */
int qpms_trans_calculator_test_sswf_lc3p(const qpms_trans_calculator *c,
    _Complex double *dest, ///< Result array, length `qpms_lMax2nelem_sc(c->lMax)`
    const _Complex double k, ///< Wavenumber
    const cart3_t r, ///< Evaluation point (excl. wavenumber)
    const qpms_bessel_t J);


#ifdef LATTICESUMS32
int qpms_trans_calculator_test_sswf_e32(const qpms_trans_calculator *c,
    _Complex double * const sigmas_total, double * serr_total,
    const double eta, const _Complex double k,
    const cart2_t b1, const cart2_t b2,
    const cart2_t beta,
    const cart3_t particle_shift,
    double maxR, double maxK,
    const qpms_ewald_part parts);
#endif // LATTICESUMS32

#endif // TRANSLATIONS_DBG_H
